@@ -15,13 +15,15 @@ from django.contrib.admin.sites import AlreadyRegistered
 
 from pages import settings
 from pages.models import Page, Content, PageAlias
-from pages.utils import has_page_add_permission, get_placeholders
 from pages.http import get_language_from_request, get_template_from_request
 
+from pages.utils import get_placeholders
+from pages.utils import has_page_add_permission, get_language_from_request
+from pages.admin.utils import get_connected, make_inline_admin
 from pages.admin import widgets
 from pages.admin.forms import PageForm
-from pages.admin.utils import get_connected, make_inline_admin
-from pages.admin.views import traduction, get_content, sub_menu, change_status, modify_content
+from pages.admin.views import traduction, get_content, sub_menu
+from pages.admin.views import change_status, modify_content, delete_content
 
 class PageAdmin(admin.ModelAdmin):
     """Page Admin class."""
@@ -31,7 +33,7 @@ class PageAdmin(admin.ModelAdmin):
     # these mandatory fields are not versioned
     mandatory_placeholders = ('title', 'slug')
     general_fields = ['title', 'slug', 'status', 'target', 'position']
-    
+
     # TODO: find solution to do this dynamically
     #if getattr(settings, 'PAGE_USE_SITE_ID'):
     general_fields.append('sites')
@@ -53,6 +55,7 @@ class PageAdmin(admin.ModelAdmin):
     if settings.PAGE_TEMPLATES:
         normal_fields.append('template')
     normal_fields.append('redirect_to')
+    normal_fields.append('redirect_to_url')
     fieldsets = (
         (_('General'), {
             'fields': general_fields,
@@ -86,6 +89,8 @@ class PageAdmin(admin.ModelAdmin):
         DEPRECATED. This function is the old way of handling URL resolution, and
         is deprecated in favor of real URL resolution -- see ``get_urls()``.
         """
+
+        # Delegate to the appropriate method, based on the URL.
         if url is None:
             return self.list_pages(request)
         elif url == 'jsi18n':
@@ -100,56 +105,41 @@ class PageAdmin(admin.ModelAdmin):
             page_id, action, content_id, language_id = url.split('/')
             return modify_content(request, unquote(page_id),
                                     unquote(content_id), unquote(language_id))
+        elif 'delete-content' in url:
+            page_id, action, language_id = url.split('/')
+            return delete_content(request, unquote(page_id), unquote(language_id))
         elif url.endswith('/sub-menu'):
             return sub_menu(request, unquote(url[:-9]))
         elif url.endswith('/move-page'):
             return self.move_page(request, unquote(url[:-10]))
-        elif url.endswith('/change-status-draft'):
-            return change_status(request, unquote(url[:-20]), Page.DRAFT)
-        elif url.endswith('/change-status-published'):
-            return change_status(request, unquote(url[:-24]), Page.PUBLISHED)
-        elif url.endswith('/change-status-hidden'):
-            return change_status(request, unquote(url[:-21]), Page.HIDDEN)
+        elif url.endswith('/change-status'):
+            page_id, action = url.split('/')
+            return change_status(request, page_id)
+
         ret = super(PageAdmin, self).__call__(request, url)
 
-        """Persist the language and template GET arguments, both on "save and
-        keep editing" and when switching language and template (which
-        also submits)
-        """
-
-        # TODO: clean that code
-        new_lang = request.GET.get('new_language', False)
-        lang = request.GET.get('language', False)
-        template = request.GET.get('template', False)
-        
-        if HttpResponseRedirect == type(ret) and (new_lang or lang or
-                                                            template):
-            for item in ret.items():
-                if 'Location' == item[0]:
-                    new_uri = item[1] + \
-                        '?language=' + request.GET.get('new_language', request.GET.get('language', '')) + \
-                        '&template=' + request.GET.get('new_template', request.GET.get('template', ''))
-                    ret = HttpResponseRedirect(new_uri)
-                    break
         return ret
 
+
     def urls(self):
         from django.conf.urls.defaults import patterns, url, include
-        
+
         # Admin-site-wide views.
         urlpatterns = patterns('',
             url(r'^$', self.list_pages, name='page-index'),
-            url(r'^(?P<page_id>[-\w]+)/traduction/(?P<language_id>[-\w]+)/$',
+            url(r'^(?P<page_id>[0-9]+)/traduction/(?P<language_id>[-\w]+)/$',
                 traduction, name='page-traduction'),
-            url(r'^(?P<page_id>[-\w]+)/get-content/(?P<content_id>[-\w]+)/$',
+            url(r'^(?P<page_id>[0-9]+)/get-content/(?P<content_id>[-\w]+)/$',
                 get_content, name='page-traduction'),
-            url(r'^(?P<page_id>[-\w]+)/modify-content/(?P<content_id>[-\w]+)/(?P<language_id>[-\w]+)/$',
+            url(r'^(?P<page_id>[0-9]+)/modify-content/(?P<content_id>[-\w]+)/(?P<language_id>[-\w]+)/$',
                 modify_content, name='page-traduction'),
-            url(r'^(?P<page_id>[-\w]+)/sub-menu/$',
+            url(r'^(?P<page_id>[0-9]+)/delete-content/(?P<language_id>[-\w]+)/$',
+                delete_content, name='page-delete_content'),
+            url(r'^(?P<page_id>[0-9]+)/sub-menu/$',
                 sub_menu, name='page-sub-menu'),
-            url(r'^(?P<page_id>[-\w]+)/move-page/$',
+            url(r'^(?P<page_id>[0-9]+)/move-page/$',
                 self.move_page, name='page-traduction'),
-            url(r'^(?P<page_id>[-\w]+)/change-status-(?P<status>[-\w]+)/$',
+            url(r'^(?P<page_id>[0-9]+)/change-status/$',
                 change_status, name='page-change-status'),
         )
         urlpatterns += super(PageAdmin, self).urls
@@ -180,7 +170,7 @@ class PageAdmin(admin.ModelAdmin):
         target = form.data.get('target', None)
         position = form.data.get('position', None)
         obj.save()
-
+        
         if target and position:
             try:
                 target = self.model.objects.get(pk=target)
@@ -215,7 +205,7 @@ class PageAdmin(admin.ModelAdmin):
     def get_fieldsets(self, request, obj=None):
         """Add fieldsets of placeholders to the list of already
         existing fieldsets.
-         """
+        """
         additional_fieldsets = []
 
         placeholder_fieldsets = []
@@ -281,7 +271,11 @@ class PageAdmin(admin.ModelAdmin):
             form.base_fields['template'].initial = force_unicode(template)
 
         for placeholder in get_placeholders(template):
-            widget = self.get_widget(placeholder.widget)()
+            widget_class = self.get_widget(placeholder.widget)
+            try:
+                widget = widget_class(language=language)
+            except TypeError:
+                widget = widget_class()
             if placeholder.parsed:
                 help_text = _('Note: This field is evaluated as template code.')
             else:
@@ -302,10 +296,11 @@ class PageAdmin(admin.ModelAdmin):
 
     def change_view(self, request, object_id, extra_context=None):
         """The 'change' admin view for the Page model."""
-        lang = get_language_from_request(request)
+        language = get_language_from_request(request)
         extra_context = {
-            'language': lang,
-            'lang': lang,
+            'language': language,
+            # don't see where it's used
+            #'lang': current_lang,
             'page_languages': settings.PAGE_LANGUAGES,
         }
         try:
@@ -320,7 +315,7 @@ class PageAdmin(admin.ModelAdmin):
             extra_context['placeholders'] = get_placeholders(template)
             extra_context['traduction_languages'] = [l for l in
                 settings.PAGE_LANGUAGES if Content.objects.get_content(obj,
-                                                            l[0], "title")]
+                                    l[0], "title") and l[0] != language]
         extra_context['page'] = obj
         return super(PageAdmin, self).change_view(request, object_id,
                                                         extra_context)
@@ -366,15 +361,27 @@ class PageAdmin(admin.ModelAdmin):
         # HACK: overrides the changelist template and later resets it to None
         if template_name:
             self.change_list_template = template_name
-        lang = get_language_from_request(request)
+        language = get_language_from_request(request)
+
+        q=request.POST.get('q', '').strip()
+
+        if q:
+            page_ids = list(set([c.page.pk for c in Content.objects.filter(body__icontains=q)]))
+            pages = Page.objects.filter(pk__in=page_ids)
+        else:
+            pages = Page.objects.root()
+
         context = {
-            'lang': lang,
+            'language': language,
             'name': _("page"),
-            'pages': Page.objects.root().order_by("tree_id"),
-            'opts': self.model._meta
+            'pages': pages,
+            'opts': self.model._meta,
+            'q': q
         }
+
         context.update(extra_context or {})
         change_list = self.changelist_view(request, context)
+
         self.change_list_template = None
         return change_list
 
