@@ -1,13 +1,15 @@
 # -*- coding: utf-8 -*-
-"""Default Page CMS widgets."""
+"""Django CMS come with a set of ready to use widgets that you can enable
+in the admin via a placeholder tag in your template."""
 from os.path import join
 from django.conf import settings
 from django.forms import TextInput, Textarea
 from django.utils.safestring import mark_safe
 from django.template import RequestContext
 from django.template.loader import render_to_string
-from pages.settings import PAGES_MEDIA_URL, PAGE_TAGGING, PAGE_TINYMCE
+from pages.settings import PAGES_MEDIA_URL, PAGE_TAGGING, PAGE_TINYMCE, PAGE_LINK_FILTER
 from pages.models import Page
+from pages.utils import get_language_from_request 
 
 if PAGE_TAGGING:
     from tagging.models import Tag
@@ -23,7 +25,11 @@ if PAGE_TAGGING:
                 'javascript/jquery.autocomplete.min.js'
             )]
 
-        def render(self, name, value, attrs=None):
+        def __init__(self, language=None, attrs=None):
+            self.language = language
+            super(AutoCompleteTagInput, self).__init__(attrs)
+
+        def render(self, name, value, language=None, attrs=None):
             rendered = super(AutoCompleteTagInput, self).render(name, value, attrs)
             page_tags = Tag.objects.usage_for_model(Page)
             context = {
@@ -45,8 +51,9 @@ class RichTextarea(Textarea):
             )]
         }
 
-    def __init__(self, attrs=None):
+    def __init__(self, language=None, attrs=None):
         attrs = {'class': 'rte'}
+        self.language = language
         super(RichTextarea, self).__init__(attrs)
 
     def render(self, name, value, attrs=None):
@@ -63,7 +70,8 @@ if PAGE_TINYMCE:
     
     class TinyMCE(tinymce_widgets.TinyMCE):
         """TinyMCE widget."""
-        def __init__(self, content_language=None, attrs=None, mce_attrs={}):
+        def __init__(self, language=None, attrs=None, mce_attrs={}):
+            self.language = language
             self.mce_attrs = mce_attrs
             self.mce_attrs.update({
                 'mode': "exact",
@@ -73,7 +81,7 @@ if PAGE_TINYMCE:
                 'theme_advanced_toolbar_location': "top",
                 'theme_advanced_toolbar_align': "left"
             })
-            super(TinyMCE, self).__init__(content_language, attrs, mce_attrs)
+            super(TinyMCE, self).__init__(language, attrs, mce_attrs)
 
 class WYMEditor(Textarea):
     """WYMEditor widget."""
@@ -81,16 +89,18 @@ class WYMEditor(Textarea):
     class Media:
         js = [join(PAGES_MEDIA_URL, path) for path in (
             'javascript/jquery.js',
+            'javascript/jquery.ui.js',
+            'javascript/jquery.ui.resizable.js',
             'wymeditor/jquery.wymeditor.js',
             'wymeditor/plugins/resizable/jquery.wymeditor.resizable.js',
         )]
-        
+       
         if "filebrowser" in getattr(settings, 'INSTALLED_APPS', []):
             js.append(join(PAGES_MEDIA_URL, 'wymeditor/plugins/filebrowser/jquery.wymeditor.filebrowser.js'))
         
 
     def __init__(self, language=None, attrs=None):
-        self.language = language or settings.LANGUAGE_CODE[:2]
+        self.language = language
         self.attrs = {'class': 'wymeditor'}
         if attrs:
             self.attrs.update(attrs)
@@ -99,12 +109,17 @@ class WYMEditor(Textarea):
     def render(self, name, value, attrs=None):
         rendered = super(WYMEditor, self).render(name, value, attrs)
         context = {
-            'page_list':Page.objects.all().order_by('tree_id'),
             'name': name,
+            'lang': self.language[:2],
             'language': self.language,
             'PAGES_MEDIA_URL': PAGES_MEDIA_URL,
         }
-        
+        context['page_link_wymeditor'] = 0
+        #if [editor for editor in PAGE_LINK_EDITOR if editor.endswith('WYMEditor')]:
+        # let's enable that by default
+        context['page_link_wymeditor'] = 1
+        context['page_list'] = Page.objects.all().order_by('tree_id','lft')
+
         context['filebrowser'] = 0
         if "filebrowser" in getattr(settings, 'INSTALLED_APPS', []):
             context['filebrowser'] = 1
@@ -169,7 +184,7 @@ class EditArea(Textarea):
     
         
     def __init__(self, language=None, attrs=None):
-        self.language = language or settings.LANGUAGE_CODE[:2]
+        self.language = language
         self.attrs = {'class': 'editarea',}
         if attrs:
             self.attrs.update(attrs)
