@@ -1,17 +1,6 @@
 # -*- coding: utf-8 -*-
-"""Django page CMS managers
-
-Manager Classes
--------------
-
-    .. class:: PageManager
-        Page manager methods
-
-    .. class:: ContentManager
-        Content manager methods
-
-"""
-import itertools
+"""Django page CMS managers."""
+import itertools, re
 from datetime import datetime
 from django.db import models, connection
 from django.contrib.sites.managers import CurrentSiteManager
@@ -20,16 +9,20 @@ from django.db.models import Q
 from django.core.cache import cache
 
 from pages import settings
+from pages.utils import normalize_url, filter_link
 
 class PageManager(models.Manager):
     """
-    Page manager provide several filters to obtain pages ``QuerySet``
+    Page manager provide several filters to obtain pages :class:`QuerySet`
     that respect the page settings.
     """
     
     def on_site(self, site_id=None):
-        """Return a ``QuerySet`` of pages that are published on the site
-        defined by the ``SITE_ID`` setting."""
+        """Return a :class:`QuerySet` of pages that are published on the site
+        defined by the ``SITE_ID`` setting.
+
+        :param site_id: specify the id of the site object to filter with.
+        """
         if settings.PAGE_USE_SITE_ID:
             if not site_id:
                 site_id = settings.SITE_ID
@@ -37,20 +30,20 @@ class PageManager(models.Manager):
         return self
 
     def root(self):
-        """Return a ``QuerySet`` of pages without parent."""
+        """Return a :class:`QuerySet` of pages without parent."""
         return self.filter(parent__isnull=True)
 
     def navigation(self):
-        """Creates a ``QuerySet`` of the published root pages."""
+        """Creates a :class:`QuerySet` of the published root pages."""
         return self.on_site().filter(
                 status=self.model.PUBLISHED).filter(parent__isnull=True)
 
     def hidden(self):
-        """Creates a ``QuerySet`` of the hidden pages."""
+        """Creates a :class:`QuerySet` of the hidden pages."""
         return self.on_site().filter(status=self.model.HIDDEN)
 
     def filter_published(self, queryset):
-        """Filter the given pages ``QuerySet`` to obtain only published
+        """Filter the given pages :class:`QuerySet` to obtain only published
         page."""
         if settings.PAGE_USE_SITE_ID:
             queryset = queryset.filter(sites=settings.SITE_ID)
@@ -68,18 +61,20 @@ class PageManager(models.Manager):
         return queryset
 
     def published(self):
-        """Creates a ``QuerySet`` of published filter."""
+        """Creates a :class:`QuerySet` of published filter."""
         return self.filter_published(self)
 
     def drafts(self):
-        """Creates a ``QuerySet`` of drafts using the page's
-        status and ``publication_date``."""
+        """Creates a :class:`QuerySet` of drafts using the page's
+        :attr:`Page.publication_date`."""
         pub = self.on_site().filter(status=self.model.DRAFT)
         if settings.PAGE_SHOW_START_DATE:
             pub = pub.filter(publication_date__gte=datetime.now())
         return pub
 
     def expired(self):
+        """Creates a :class:`QuerySet` of expired using the page's
+        :attr:`Page.publication_end_date`."""
         return self.on_site().filter(
             publication_end_date__lte=datetime.now())
 
@@ -106,43 +101,66 @@ class ContentManager(models.Manager):
     """Content manager methods"""
 
     def sanitize(self, content):
-        """Sanitize the content to avoid XSS"""
+        """Sanitize a string in order to avoid possible XSS."""
         import html5lib
         from html5lib import sanitizer
         p = html5lib.HTMLParser(tokenizer=sanitizer.HTMLSanitizer)
         # we need to remove <html><head/><body>...</body></html>
         return p.parse(content).toxml()[19:-14]
 
-    def set_or_create_content(self, page, language, cnttype, body):
-        """Set or create a content for a particular page and language."""
+    def set_or_create_content(self, page, language, ctype, body):
+        """Set or create a :class:`Content <pages.models.Content>` for a
+        particular page and language.
+
+        :param page: the concerned page object.
+        :param language: the wanted language.
+        :param ctype: the content type.
+        :param body: the content of the Content object.
+        """
         if settings.PAGE_SANITIZE_USER_INPUT:
-            body = self.sanitize(body)
+            body = self.sanitize(body)       
         try:
             content = self.filter(page=page, language=language,
-                                  type=cnttype).latest('creation_date')
+                                  type=ctype).latest('creation_date')
             content.body = body
         except self.model.DoesNotExist:
             content = self.model(page=page, language=language, body=body,
-                                 type=cnttype)
+                                 type=ctype)
         content.save()
         return content
 
-    def create_content_if_changed(self, page, language, cnttype, body):
-        """Set or create a content for a particular page and language"""
+    def create_content_if_changed(self, page, language, ctype, body):
+        """Create a :class:`Content <pages.models.Content>` for a particular
+        page and language only if the content has changed from the last
+        time.
+
+        :param page: the concerned page object.
+        :param language: the wanted language.
+        :param ctype: the content type.
+        :param body: the content of the Content object.
+        """
         if settings.PAGE_SANITIZE_USER_INPUT:
             body = self.sanitize(body)
         try:
             content = self.filter(page=page, language=language,
-                                  type=cnttype).latest('creation_date')
+                                  type=ctype).latest('creation_date')
             if content.body == body:
                 return content
         except self.model.DoesNotExist:
             pass
-        content = self.create(page=page, language=language, body=body, type=cnttype)
+        content = self.create(page=page, language=language, body=body,
+                type=ctype)
 
     def get_content(self, page, language, ctype, language_fallback=False):
-        """Gets the latest ``Content`` for a particular page and language.
-        Falls back to another language if wanted."""
+        """Gets the latest :class:`Content <pages.models.Content>`
+        for a particular page and language. Falls back to another
+        language if wanted.
+
+        :param page: the concerned page object.
+        :param language: the wanted language.
+        :param ctype: the content type.
+        :param language_fallback: fallback to another language if ``True``.
+        """
         PAGE_CONTENT_DICT_KEY = "page_content_dict_%s_%s"
         if not language:
             language = settings.PAGE_DEFAULT_LANGUAGE
@@ -154,24 +172,31 @@ class ContentManager(models.Manager):
             content_dict = {}
             for lang in settings.PAGE_LANGUAGES:
                 try:
-                    content = self.filter(language=lang[0], type=ctype, page=page).latest()
+                    content = self.filter(
+                        language=lang[0],
+                        type=ctype,
+                        page=page
+                    ).latest()
                     content_dict[lang[0]] = content.body
                 except self.model.DoesNotExist:
                     content_dict[lang[0]] = ''
             cache.set(PAGE_CONTENT_DICT_KEY % (page.id, ctype), content_dict)
 
         if language in content_dict and content_dict[language]:
-            return content_dict[language]
+            return filter_link(content_dict[language], page, language, ctype)
 
         if language_fallback:
             for lang in settings.PAGE_LANGUAGES:
-                if content_dict[lang[0]]:
-                    return content_dict[lang[0]]
+                if lang[0] in content_dict:
+                    return filter_link(content_dict[lang[0]], page, lang[0], ctype)
         return ''
 
     def get_content_slug_by_slug(self, slug):
-        """Returns the latest ``Content`` slug object that match the given
-        slug for the current site domain."""
+        """Returns the latest :class:`Content <pages.models.Content>`
+        slug object that match the given slug for the current site domain.
+
+        :param slug: the wanted slug.
+        """
         content = self.filter(type='slug', body=slug)
         if settings.PAGE_USE_SITE_ID:
             content = content.filter(page__sites__id=settings.SITE_ID)
@@ -183,7 +208,10 @@ class ContentManager(models.Manager):
             return content
 
     def get_page_ids_by_slug(self, slug):
-        """Return all page id matching the given slug."""
+        """Return all page's id matching the given slug.
+
+        :param slug: the wanted slug.
+        """
         sql = '''SELECT pages_content.page_id,
             MAX(pages_content.creation_date)
             FROM pages_content WHERE (pages_content.type = %s
@@ -198,8 +226,12 @@ class PagePermissionManager(models.Manager):
     """Hierachic page permission manager."""
 
     def get_page_id_list(self, user):
-        """Give a list of ``Page`` ids where the user has rights or the string
-        "All" if the user has all rights."""
+        """Give a list of :class:`Page <pages.models.Page>` ids where the
+        user has rights or the string
+        "All" if the user has all rights.
+
+        :param user: the interested user.
+        """
         if user.is_superuser:
             return 'All'
         id_list = []
@@ -216,15 +248,18 @@ class PagePermissionManager(models.Manager):
 
 class PageAliasManager(models.Manager):
     """PageAlias manager."""
-    def from_path(self, request, path=None, lang=None):
+    def from_path(self, request, path, lang):
         """
-        Resolve a request to an alias. returns a ``PageAlias`` object or None
+        Resolve a request to an alias. returns a :class:`PageAlias <pages.models.PageAlias>`
         if the url matches no page at all. The aliasing system supports plain
         aliases (``/foo/bar``) as well as aliases containing GET parameters
         (like ``index.php?page=foo``).
+
+        :param request: the request object
+        :param path: the complete path to the page
+        :param lang: not used
         """
-        from pages.utils import normalize_url
-        from pages.models import Page,PageAlias
+        from pages.models import PageAlias
 
         url = normalize_url(path)
         # §1: try with complete query string
