@@ -1,18 +1,6 @@
 # -*- coding: utf-8 -*-
-"""Django page CMS models
-
-Model Classes
--------------
-
-    .. class:: Page
-        A simple hierarchical page model
-
-    .. class:: Content
-        A block of content, tied to a page, for a particular language
-
-"""
+"""Django page CMS models."""
 from datetime import datetime
-
 from django.db import models
 from django.contrib.auth.models import User
 from django.template.defaultfilters import slugify
@@ -21,17 +9,39 @@ from django.utils.safestring import mark_safe
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.contrib.sites.models import Site
-
 import mptt
-from pages import settings
 from pages.utils import get_placeholders, normalize_url
-from pages.managers import PageManager, ContentManager, PagePermissionManager, PageAliasManager
-
+from pages.managers import PageManager, ContentManager
+from pages.managers import PagePermissionManager, PageAliasManager
+from pages.lib.BeautifulSoup import BeautifulSoup
+from pages import settings
 
 class Page(models.Model):
     """
     This model contain the status, dates, author, template.
-    The real content of the page can be found in the ``Content`` model."""
+    The real content of the page can be found in the
+    :class:`Content <pages.models.Content>` model.
+
+    .. attribute:: creation_date
+    When the page has been created.
+
+    .. attribute:: publication_date
+    When the page should be visible.
+
+    .. attribute:: publication_end_date
+    When the publication of this page end.
+
+    .. attribute:: last_modification_date
+    Last time this page has been modified.
+
+    .. attribute:: status
+    The current status of the page. Could be DRAFT, PUBLISHED,
+    EXPIRED or HIDDEN. You should the property :attr:`calculated_status` if
+    you want that the dates are taken in account.
+
+    .. attribute:: template
+    A string containing the name of the template file for this page.
+    """
     
     # some class constants to refer to, e.g. Page.DRAFT
     DRAFT = 0
@@ -49,8 +59,10 @@ class Page(models.Model):
     #PAGE_TEMPLATE_KEY = "page_%d_template"
     #PAGE_CHILDREN_KEY = "page_children_%d_%d"
     PAGE_CONTENT_DICT_KEY = "page_content_dict_%d_%s"
+    PAGE_BROKEN_LINK_KEY = "page_broken_link_%s"
 
     author = models.ForeignKey(User, verbose_name=_('author'))
+    
     parent = models.ForeignKey('self', null=True, blank=True, 
             related_name='children', verbose_name=_('parent'))
     creation_date = models.DateTimeField(_('creation date'), editable=False, 
@@ -72,7 +84,8 @@ class Page(models.Model):
     sites = models.ManyToManyField(Site, default=[settings.SITE_ID], 
             help_text=_('The site(s) the page is accessible at.'),
             verbose_name=_('sites'))
-    
+    redirect_to_url = models.CharField(max_length=200, null=True, blank=True)
+
     redirect_to = models.ForeignKey('self', null=True, blank=True,
             related_name='redirected_pages')
     
@@ -84,6 +97,7 @@ class Page(models.Model):
         tags = fields.TagField(null=True)
 
     class Meta:
+        """Make sure the ordering is correct."""
         ordering = ['tree_id', 'lft']
         verbose_name = _('page')
         verbose_name_plural = _('pages')
@@ -103,28 +117,32 @@ class Page(models.Model):
             else:
                 self.publication_date = None
         self.last_modification_date = datetime.now()
+        # let's assume there is no more broken links after a save
+        cache.delete(self.PAGE_BROKEN_LINK_KEY % self.id)
         super(Page, self).save(*args, **kwargs)
 
-    def get_calculated_status(self):
-        """get the calculated status of the page based on
-        ``published_date``, ``published_end_date``, and ``status``."""
+    def _get_calculated_status(self):
+        """Get the calculated status of the page based on
+        :attr:`Page.publication_date`,
+        :attr:`Page.publication_end_date`,
+        and :attr:`Page.status`."""
         if settings.PAGE_SHOW_START_DATE and self.publication_date:
             if self.publication_date > datetime.now():
                 return self.DRAFT
-        
+
         if settings.PAGE_SHOW_END_DATE and self.publication_end_date:
             if self.publication_end_date < datetime.now():
                 return self.EXPIRED
 
         return self.status
-    calculated_status = property(get_calculated_status)
+    calculated_status = property(_get_calculated_status)
 
     def get_children_for_frontend(self):
-        """Creates a ``QuerySet`` of published children page"""
+        """Return a :class:`QuerySet` of published children page"""
         return Page.objects.filter_published(self.get_children())
 
-    def invalidate(self, language_code=None):
-        """Invalidate this page and it's descendants."""
+    def invalidate(self):
+        """Invalidate cached data for this page."""
 
         cache.delete(self.PAGE_LANGUAGES_KEY % (self.id))
         #cache.delete(self.PAGE_TEMPLATE_KEY % (self.id))
@@ -166,7 +184,10 @@ class Page(models.Model):
 
     def get_absolute_url(self, language=None):
         """Return the absolute page url. Add the language prefix if
-        ``PAGE_USE_LANGUAGE_PREFIX`` setting is set to **True***."""
+        ``PAGE_USE_LANGUAGE_PREFIX`` setting is set to ``True``.
+
+        :param language: the wanted url language.
+        """
         url = reverse('pages-root')
         if settings.PAGE_USE_LANGUAGE_PREFIX:
             url += str(language) + '/'
@@ -192,7 +213,8 @@ class Page(models.Model):
         """
         Return the slug of the page depending on the given language.
 
-        If fallback is **True**, the slug will also be searched in other
+        :param language: wanted language, if not defined default is used.
+        :param fallback: if ``True``, the slug will also be searched in other \
         languages.
         """
         
@@ -205,7 +227,8 @@ class Page(models.Model):
         """
         Return the title of the page depending on the given language.
 
-        If fallback is **True**, the title will also be searched in other
+        :param language: wanted language, if not defined default is used.
+        :param fallback: if ``True``, the slug will also be searched in other \
         languages.
         """
         if not language:
@@ -216,8 +239,9 @@ class Page(models.Model):
 
     def get_template(self):
         """
-        get the template of this page if defined or if closer parent if
-        defined or DEFAULT_PAGE_TEMPLATE otherwise
+        Get the :attr:`template <Page.template>` of this page if
+        defined or the closer parent's one if defined
+        or :attr:`pages.settings.DEFAULT_PAGE_TEMPLATE` otherwise.
         """
         if self.template:
             return self.template
@@ -234,21 +258,20 @@ class Page(models.Model):
         return template
 
     def get_template_name(self):
+        """
+        Get the template name of this page if defined or if a closer
+        parent has a defined template or
+        :data:`pages.settings.DEFAULT_PAGE_TEMPLATE` otherwise.
+        """
         template = self.get_template()
         for  t in settings.PAGE_TEMPLATES:
             if t[0] == template:
                 return t[1]
         return template
-        
-    def traductions(self):
-        langs = ""
-        for lang in self.get_languages():
-            langs += '%s, ' % lang
-        return langs[0:-2]
 
     def has_page_permission(self, request):
         """
-        Return true if the current user has permission on the page.
+        Return ``True`` if the current user has permission on the page.
         Return the string 'All' if the user has all rights.
         """
         if not settings.PAGE_PERMISSION:
@@ -261,9 +284,15 @@ class Page(models.Model):
                 return True
             return False
 
+    def has_broken_link(self):
+        return cache.get(self.PAGE_BROKEN_LINK_KEY % self.id)
+
     def valid_targets(self, perms="All"):
-        """Return a ``QuerySet`` of valid targets for moving a page into the
-        tree."""
+        """Return a :class:`QuerySet` of valid targets for moving a page into the
+        tree.
+
+        :param perms: the level of permission of the concerned user.
+        """
         exclude_list = [self.id]
         for p in self.get_descendants():
             exclude_list.append(p.id)
@@ -273,14 +302,14 @@ class Page(models.Model):
         else:
             return Page.objects.exclude(id__in=exclude_list)
 
-    def with_level(self):
+    def slug_with_level(self, language=None):
         """Display the slug of the page prepended with insecable
         spaces equal to the level of page in the hierarchy."""
         level = ''
         if self.level:
             for n in range(0, self.level):
                 level += '&nbsp;&nbsp;&nbsp;'
-        return mark_safe(level + self.slug())
+        return mark_safe(level + self.slug(language))
         
     def margin_level(self):
         return self.level * 2
