@@ -136,14 +136,29 @@ PAGE_SHOW_START_DATE = getattr(settings, 'PAGE_SHOW_START_DATE', False)
 PAGE_SHOW_END_DATE = getattr(settings, 'PAGE_SHOW_END_DATE', False)
 
 # ``PAGE_CONNECTED_MODELS`` allows you to specify a model and form for this 
-# model into your settings to get an automatic form to create and directly link 
-# a new instance of this model with your page in the admin.  
+# model into your settings to get an automatic form to create
+# and directly link a new instance of this model with your page in the admin.
 #
 # Here is an example:
 #
 # PAGE_CONNECTED_MODELS = [
-#     {'model':'documents.models.Document','form':'documents.models.DocumentForm'},
+#     {'model':'documents.models.Document',
+#        'form':'documents.models.DocumentForm'},
 # ]
 #
 PAGE_CONNECTED_MODELS = getattr(settings, 'PAGE_CONNECTED_MODELS', False)
 
+# pagelink url validator agent
+PAGE_URL_VALIDATOR_USER_AGENT = getattr(settings, 'URL_VALIDATOR_USER_AGENT', False)
+
+# The page link filter enable a output filter on you content links. The goal is
+# to transform special page class into real links at the last moment. This ensure
+# that even if you have moved a page, the URL will be correct.
+PAGE_LINK_FILTER = getattr(settings, 'PAGE_LINK_FILTER', True)
+
+# pagelink url validator user agent
+PAGE_URL_VALIDATOR_USER_AGENT = getattr(settings, 'URL_VALIDATOR_USER_AGENT', False)
+
+# This setting is a function that can be defined if you need to pass extra context
+# data to the pages templates.
+PAGE_EXTRA_CONTEXT = getattr(settings, 'PAGE_EXTRA_CONTEXT', None)
\ No newline at end of file
