@@ -15,8 +15,8 @@ register = template.Library()
 
 PLACEHOLDER_ERROR = _("[Placeholder %(name)s had syntax error: %(error)s]")
 
-def get_content(context, page, content_type, lang, fallback=True):
-    """Helper function used by placeholder nodes"""
+def _get_content(context, page, content_type, lang, fallback=True):
+    """Helper function used by placeholder nodes."""
     if not page:
         return ''
 
@@ -36,21 +36,33 @@ def get_content(context, page, content_type, lang, fallback=True):
 """Filters"""
 
 def has_content_in(page, language):
-    """Tell if the page has any content in a particular language"""
+    """Fitler that return ``True`` if the page has any content in a
+    particular language.
+
+    :param page: the current page
+    :param language: the language you want to look at
+    """
     return Content.objects.filter(page=page, language=language).count() > 0
 register.filter(has_content_in)
 
 def has_permission(page, request):
-    """Tell if a user has permissions on the page according to the request
-       object"""
+    """Tell if a user has permissions on the page.
+
+    :param page: the current page
+    :param request: the request object where the user is extracted
+    """
     return page.has_page_permission(request)
 register.filter(has_permission)
 
 """Inclusion tags"""
 
 def pages_menu(context, page, url='/'):
-    """Render a nested list of all children of the given page, including
-    this page"""
+    """Render a nested list of all the descendents of the given page,
+    including this page.
+
+    :param page: the page where to start the menu from.
+    :param url: not used anymore.
+    """
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
     path = context.get('path', None)
     site_id = None
@@ -63,7 +75,12 @@ pages_menu = register.inclusion_tag('pages/menu.html',
 
 def pages_sub_menu(context, page, url='/'):
     """Get the root page of the given page and
-    render a nested list of all root's children pages"""
+    render a nested list of all root's children pages.
+    Good for rendering a secondary menu.
+
+    :param page: the page where to start the menu from.
+    :param url: not used anymore.
+    """
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
     path = context.get('path', None)
     root = page.get_root()
@@ -75,7 +92,7 @@ pages_sub_menu = register.inclusion_tag('pages/sub_menu.html',
                                         takes_context=True)(pages_sub_menu)
 
 def pages_admin_menu(context, page, url='', level=None):
-    """Render the admin table of pages"""
+    """Render the admin table of pages."""
     request = context['request']
     
     if "tree_expanded" in request.COOKIES:
@@ -99,35 +116,53 @@ pages_admin_menu = register.inclusion_tag('admin/pages/page/menu.html',
 def show_content(context, page, content_type, lang=None, fallback=True):
     """Display a content type from a page.
     
-    eg: {% show_content page_object "title" %}
+    Example::
+
+        {% show_content page_object "title" %}
     
-    You can also use the slug of a page
+    You can also use the slug of a page::
     
-    eg: {% show_content "my-page-slug" "title" %}
-    
-    Keyword arguments:
-    page -- the page object
-    args -- content_type used by a placeholder
-    lang -- the wanted language (default None, use the request object to know)
-    fallback -- use fallback content
+        {% show_content "my-page-slug" "title" %}
+
+    :param page: the page object
+    :param content_type: content_type used by a placeholder
+    :param lang: the wanted language (default None, use the request object to know)
+    :param fallback: use fallback content
     """
-    return {'content':get_content(context, page, content_type, lang,
+    return {'content':_get_content(context, page, content_type, lang,
                                                                 fallback)}
 show_content = register.inclusion_tag('pages/content.html',
                                       takes_context=True)(show_content)
 
+def show_slug_with_level(context, page, lang=None, fallback=True):
+    """Display slug with level by language."""
+    if not page:
+        return ''
+    if not lang:
+        lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+
+    if not page:
+        return ''
+
+    return {'content': page.slug_with_level(lang)}
+show_slug_with_level = register.inclusion_tag('pages/content.html',
+                                      takes_context=True)(show_slug_with_level)
+
+
 def show_absolute_url(context, page, lang=None):
     """Show the url of a page in the right language
 
-    eg: {% show_absolute_url page_object %}
+    Example ::
+    
+        {% show_absolute_url page_object %}
 
-    You can also use the slug of a page
-
-    eg: {% show_absolute_url "my-page-slug" %}
+    You can also use the slug of a page::
+    
+        {% show_absolute_url "my-page-slug" %}
 
     Keyword arguments:
-    page -- the page object or a slug string
-    lang -- the wanted language (defaults to None, uses request object else)
+    :param page: the page object or a slug string
+    :param lang: the wanted language (defaults to `settings.PAGE_DEFAULT_LANGUAGE`)
     """
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
     # if the page is a SafeUnicode, try to use it like a slug
@@ -144,7 +179,7 @@ show_absolute_url = register.inclusion_tag('pages/content.html',
 
 def show_revisions(context, page, content_type, lang=None):
     """Render the last 10 revisions of a page content with a list using
-        the pages/revisions.html template"""
+        the ``pages/revisions.html`` template"""
     if not settings.PAGE_CONTENT_REVISION:
         return {'revisions':None}
     revisions = Content.objects.filter(page=page, language=lang,
@@ -155,6 +190,54 @@ def show_revisions(context, page, content_type, lang=None):
 show_revisions = register.inclusion_tag('pages/revisions.html',
                                         takes_context=True)(show_revisions)
 
+def pages_dynamic_tree_menu(context, page, url='/'):
+    """
+    Render a "dynamic" tree menu, with all nodes expanded which are either
+    ancestors or the current page itself.
+
+    Override ``pages/dynamic_tree_menu.html`` if you want to change the
+    design.
+
+    :param page: the current page
+    :param url: not used anymore
+    """
+    request = context['request']
+    site_id = None
+    children = None
+    lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    if 'current_page' in context:
+        current_page = context['current_page']
+        # if this node is expanded, we also have to render its children
+        # a node is expanded if it is the current node or one of its ancestors
+        if page.lft <= current_page.lft and page.rght >= current_page.rght:
+            children = page.get_children_for_frontend()
+    return locals()
+pages_dynamic_tree_menu = register.inclusion_tag(
+    'pages/dynamic_tree_menu.html',
+    takes_context=True
+)(pages_dynamic_tree_menu)
+
+def pages_breadcrumb(context, page, url='/'):
+    """
+    Render a breadcrumb like menu.
+
+    Override ``pages/breadcrumb.html`` if you want to change the
+    design.
+    
+    :param page: the current page
+    :param url: not used anymore
+    """
+    lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    request = context['request']
+    site_id = None
+    pages = page.get_ancestors()
+    return locals()
+pages_breadcrumb = register.inclusion_tag(
+    'pages/breadcrumb.html',
+    takes_context=True
+)(pages_breadcrumb)
+
+
 """Tags"""
 
 class GetContentNode(template.Node):
@@ -169,7 +252,7 @@ class GetContentNode(template.Node):
             lang = None
         else:
             lang = self.lang.resolve(context)
-        context[self.varname] = get_content(context,
+        context[self.varname] = _get_content(context,
             self.page.resolve(context),
             self.content_type.resolve(context),
             lang)
@@ -178,18 +261,22 @@ class GetContentNode(template.Node):
 def do_get_content(parser, token):
     """Store a content type from a page into a context variable.
 
-    eg: {% get_content page_object "title" as content %}
+    Example::
+    
+        {% get_content page_object "title" as content %}
 
-    You can also use the slug of a page
+    You can also use the slug of a page::
+    
+        {% get_content "my-page-slug" "title" as content %}
 
-    eg: {% get_content "my-page-slug" "title" as content %}
+    Syntax::
+    
+        {% get_content page type [lang] as name %}
 
-    Syntax: {% get_content page type [lang] as name %}
-    Arguments:
-    page -- the page object
-    type -- content_type used by a placeholder
-    name -- name of the context variable to store the content in
-    lang -- the wanted language
+    :param page: the page object
+    :param type: content_type used by a placeholder
+    :param name: name of the context variable to store the content in
+    :param lang: the wanted language
     """
     bits = token.split_contents()
     if not 5 <= len(bits) <= 6:
@@ -210,27 +297,25 @@ do_get_content = register.tag('get_content', do_get_content)
 class LoadPagesNode(template.Node):
     """Load page node."""
     def render(self, context):
-        request = context['request']
-        try:
-            page_dict = details(request, only_context=True)
-            context.update(page_dict)
-        except Exception, e:
-            context.update({'current_page': Page()})
-            pass
+        if 'pages' not in context:
+            pages = Page.objects.navigation().order_by("tree_id")
+            context.update({'pages': pages})
+        if 'current_page' not in context:
+            context.update({'current_page':None})
         return ''
 
 def do_load_pages(parser, token):
     """Load the navigation pages, lang, and current_page variables into the
     current context.
 
-    eg:
-    <ul>
-        {% load_pages %}
-        {% for page in pages %}
-            {% pages_menu page %}
-        {% endfor %}
-    </ul>
-
+    Example::
+    
+        <ul>
+            {% load_pages %}
+            {% for page in pages %}
+                {% pages_menu page %}
+            {% endfor %}
+        </ul>
     """
     return LoadPagesNode()
 do_load_pages = register.tag('load_pages', do_load_pages)
@@ -240,17 +325,23 @@ class PlaceholderNode(template.Node):
     """This template node is used to output page content and
     also in the admin to dynamically generate input fields.
 
-    Keyword arguments:
-    name -- the name of the placeholder you want to show/create
-    page -- The optional page object  
-    widget -- the widget you want to use in the admin interface. Take
-        a look into pages.admin.widgets to see which widgets are available.
-    parsed -- If the "parsed" word is given, the content of the
+    :param name: the name of the placeholder you want to show/create
+    :param page: the optional page object
+    :param widget: the widget you want to use in the admin interface. Take
+        a look into :mod:`pages.admin.widgets` to see which widgets
+        are available.
+    :param parsed: if the ``parsed`` word is given, the content of the
         placeholder is evaluated as template code, within the current context.
+    :param as_varname: if ``as_varname`` is defined, no value will be returned.
+        A variable will be created in the context with the defined name.
     """
     def handle_token(cls, parser, token):
-        # {% placeholder <name> [on <page>] [with <widget>] [parsed] [as
-        # <varname>] %}
+        """
+        Method that parse the template tag syntax following this syntax::
+
+            {% placeholder <name> [on <page>] [with <widget>] \
+[parsed] [as <varname>] %}
+        """
         bits = token.split_contents()
         count = len(bits)
         error_string = '%r tag requires at least one argument' % bits[0]
@@ -292,6 +383,9 @@ class PlaceholderNode(template.Node):
     def render(self, context):
         if not self.page in context:
             return ''
+        # current_page can be set to None
+        if not context[self.page]:
+            return ''
 
         lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
         content = Content.objects.get_content(context[self.page], lang,
@@ -348,32 +442,4 @@ def do_placeholder(parser, token):
 
 register.tag('placeholder', do_placeholder)
 
-def pages_dynamic_tree_menu(context, page, url='/'):
-    """
-    render a "dynamic" tree menu, with all nodes expanded which are either
-    ancestors or the current page itself. 
-    """
-    request = context['request']
-    site_id = None
-    children = None
-    if 'current_page' in context:
-        current_page = context['current_page']
-        # if this node is expanded, we also have to render its children
-        # a node is expanded if it is the current node or one of its ancestors
-        if page.lft <= current_page.lft and page.rght >= current_page.rght:
-            children = page.get_children_for_frontend() 
-    return locals()
-pages_dynamic_tree_menu = register.inclusion_tag(
-    'pages/dynamic_tree_menu.html',
-    takes_context=True
-)(pages_dynamic_tree_menu)
 
-def pages_breadcrumb(context, page, url='/'):
-    request = context['request']
-    site_id = None
-    pages = page.get_ancestors()
-    return locals()
-pages_breadcrumb = register.inclusion_tag(
-    'pages/breadcrumb.html',
-    takes_context=True
-)(pages_breadcrumb)
