@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from django.conf.urls.defaults import *
 from pages.views import details
 from pages import settings
@@ -7,11 +8,13 @@ urlpatterns = patterns('',
     url(r'^$', details, name='pages-root'),
 )
 
-if settings.PAGE_UNIQUE_SLUG_REQUIRED:
+if settings.PAGE_USE_LANGUAGE_PREFIX:
     urlpatterns += patterns('',
-        url(r'^.*?/?(?P<slug>[-\w]+)/$', details, name='pages-details-by-slug'),
+        url(r'^(?P<lang>[-\w]+)/(?P<path>.*)$', details,
+            name='pages-details-by-path'),
     )
 else:
     urlpatterns += patterns('',
-        url(r'^.*?(?P<page_id>[0-9]+)/$', details, name='pages-details-by-id'),
-    )
+        url(r'^(?P<path>.*)$', details,
+            name='pages-details-by-path'),
+    )
\ No newline at end of file
